@@ -88,7 +88,7 @@ impl Compiler {
                         &format!(
                             "func while_{chars}\n\
                              execute if {condition} run call {subfolder}condition_{chars}\n\
-                             endfunc\n",
+                             end\n",
                             chars = chars,
                             condition = condition,
                             subfolder = subfolder
@@ -97,9 +97,9 @@ impl Compiler {
                     Token::WhileContents(contents) => new_contents.push_str(
                         &format!(
                             "func condition_{chars}\n\
-                             {contents}\n\
-                             call {subfolder}while_{chars}\n\
-                             endfunc\n",
+                         {contents}\n\
+                         call {subfolder}while_{chars}\n\
+                         end\n",
                             chars = chars,
                             contents = contents,
                             subfolder = subfolder
@@ -107,11 +107,11 @@ impl Compiler {
                     ),
                     Token::EndWhileLoop => {
                         new_contents.push_str(&format!("call {}while_{}\n", subfolder, chars)[..]);
+
                         chars = Compiler::get_chars();
 
                         // Tokenize new contents
                         let tks = Compiler::new(new_contents.clone(), false).tokenize(false);
-
                         new_contents = String::new();
 
                         // When gettings indexes in the new tokens vector,
