@@ -28,23 +28,16 @@ impl Compiler {
         let mut comment = false;
         let mut building_first_token = true;
         let mut current_token = String::new();
-        let mut last_char = '\n';
 
         let mut building_while = false;
         let mut building_while_condition = false;
         let mut while_lines: Vec<String> = Vec::new();
         let mut while_line = String::new();
+        let mut escaped = false;
 
         let mut building_for = Token::None;
         let mut params_left = 0;
 
-        macro_rules! next_char {
-            () => {
-                last_char = self.current_char;
-                self.next_char();
-            };
-        }
-
         macro_rules! set_building {
             ($token: expr, $params: expr) => {{
                 current_token = String::new();
@@ -52,7 +45,7 @@ impl Compiler {
                 building_first_token = false;
                 building_for = $token;
                 params_left = $params;
-                next_char!();
+                self.next_char();
             }};
         }
 
@@ -67,9 +60,17 @@ impl Compiler {
                     tokens.push($token);
                     current_token = String::new();
                     params_left -= 1;
+                    escaped = false;
                     true
                 } else {
-                    current_token.push(self.current_char);
+                    // A % at the beginning escapes a keyword
+                    if self.current_char != '%'
+                        || self.current_char == '%' && (!current_token.is_empty() || escaped)
+                    {
+                        current_token.push(self.current_char);
+                    } else {
+                        escaped = true;
+                    }
                     false
                 }
             };
@@ -139,7 +140,7 @@ impl Compiler {
 
         while self.current_char != '\u{0}' {
             while comment {
-                next_char!();
+                self.next_char();
                 if self.current_char == '\n' {
                     comment = false;
                 }
@@ -147,7 +148,10 @@ impl Compiler {
 
             if get_definitions && !tokens.is_empty() {
                 match tokens.last().unwrap() {
-                    Token::DefineReplace | Token::ReplaceName(_) | Token::ReplaceContents(_) => {}
+                    Token::DefineReplace
+                    | Token::ReplaceName(_)
+                    | Token::ReplaceContents(_)
+                    | Token::NewLine => {}
                     _ => break,
                 }
             }
@@ -155,11 +159,11 @@ impl Compiler {
             if building_while {
                 current_token.push(self.current_char);
                 if building_while_condition {
-                    if self.current_char.is_whitespace() {
+                    if self.current_char == '\n' {
                         add_token!(Token::WhileCondition(current_token.trim().into()));
                         building_while_condition = false;
                     }
-                } else if current_token != ":endwhile" {
+                } else if current_token.trim() != "end" {
                     while_line.push(self.current_char);
                     if self.current_char == '\n' {
                         current_token = String::new();
@@ -168,11 +172,13 @@ impl Compiler {
                     }
                 } else {
                     building_while = false;
+                    building_first_token = true;
                     tokens.push(Token::WhileContents(while_lines.join("\n")));
                     tokens.push(Token::EndWhileLoop);
                     current_token = String::new();
                     while_line = String::new();
                     while_lines = Vec::new();
+                    self.next_char();
                 }
             }
 
@@ -182,7 +188,7 @@ impl Compiler {
                         match &current_token[..] {
                             "func" => set_building!(Token::DefineFunc, 1),
                             "tag" => set_building!(Token::Tag, 1),
-                            "endfunc" => no_args_add!(Token::EndFunc),
+                            "end" => no_args_add!(Token::EndFunc),
                             "var" => set_building!(Token::Var, 3),
                             "obj" => set_building!(Token::Objective, 2),
                             "sobj" => set_building!(Token::SetObjective, 4),
@@ -199,21 +205,30 @@ impl Compiler {
                             }
                             "sbop" => no_args_add!(Token::ScoreboardOperation),
                             "delvar" | "delobj" => set_building!(Token::DeleteVar, 1),
-                            _ => no_args_add!(Token::NonDatabind(format!("{} ", current_token))),
+                            _ => {
+                                if current_token.starts_with('%') {
+                                    no_args_add!(Token::NonDatabind(format!(
+                                        "{} ",
+                                        current_token.strip_prefix('%').unwrap()
+                                    )));
+                                } else {
+                                    no_args_add!(Token::NonDatabind(format!("{} ", current_token)));
+                                }
+                            }
                         };
                         if self.current_char == '\n' {
                             tokens.push(Token::NewLine);
                         }
                     } else {
-                        next_char!();
+                        self.next_char();
                     }
                 } else {
-                    if last_char == '\n' && self.current_char == '#' {
+                    if self.current_char == '#' && current_token.is_empty() {
                         comment = true;
                         continue;
                     }
                     current_token.push(self.current_char);
-                    next_char!();
+                    self.next_char();
                 }
             } else {
                 match building_for {
@@ -274,7 +289,7 @@ impl Compiler {
                 if self.current_char == '\n' {
                     tokens.push(Token::NewLine);
                 }
-                next_char!();
+                self.next_char();
             }
         }
 
